@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -27,8 +27,8 @@ OF_ASSUME_NONNULL_BEGIN
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern OFIRI *OFArchiveIRIHandlerIRIForFileInArchive(OFString *, OFString *,
-    OFIRI *);
+extern OFIRI *_OFArchiveIRIHandlerIRIForFileInArchive(OFString *, OFString *,
+    OFIRI *) OF_VISIBILITY_HIDDEN;
 #ifdef __cplusplus
 }
 #endif
