@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -25,6 +25,7 @@
 #include <math.h>
 
 #import "OFString+JSONParsing.h"
+#import "OFString+Private.h"
 #import "OFArray.h"
 #import "OFDictionary.h"
 #import "OFNumber.h"
@@ -136,9 +137,34 @@ parseUnicodeEscape(const char *pointer, const char *stop)
 			return 0xFFFF;
 	}
 
-	if (ret == 0)
+	return ret;
+}
+
+static inline OFChar16
+parseHexEscape(const char *pointer, const char *stop)
+{
+	OFChar16 ret = 0;
+
+	if (pointer + 3 >= stop)
 		return 0xFFFF;
 
+	if (pointer[0] != '\\' || pointer[1] != 'x')
+		return 0xFFFF;
+
+	for (uint8_t i = 0; i < 2; i++) {
+		char c = pointer[i + 2];
+		ret <<= 4;
+
+		if (c >= '0' && c <= '9')
+			ret |= c - '0';
+		else if (c >= 'a' && c <= 'f')
+			ret |= c + 10 - 'a';
+		else if (c >= 'A' && c <= 'F')
+			ret |= c + 10 - 'A';
+		else
+			return 0xFFFF;
+	}
+
 	return ret;
 }
 
@@ -189,6 +215,10 @@ parseString(const char **pointer, const char *stop, size_t *line)
 				buffer[i++] = '\t';
 				(*pointer)++;
 				break;
+			case '0':
+				buffer[i++] = '\0';
+				(*pointer)++;
+				break;
 			/* Parse Unicode escape sequence */
 			case 'u':;
 				OFChar16 c1, c2;
@@ -209,7 +239,7 @@ parseString(const char **pointer, const char *stop, size_t *line)
 
 				/* Normal character */
 				if ((c1 & 0xFC00) != 0xD800) {
-					l = OFUTF8StringEncode(c1, buffer + i);
+					l = _OFUTF8StringEncode(c1, buffer + i);
 					if (l == 0) {
 						OFFreeMemory(buffer);
 						return nil;
@@ -235,7 +265,7 @@ parseString(const char **pointer, const char *stop, size_t *line)
 				c = (((c1 & 0x3FF) << 10) |
 				    (c2 & 0x3FF)) + 0x10000;
 
-				l = OFUTF8StringEncode(c, buffer + i);
+				l = _OFUTF8StringEncode(c, buffer + i);
 				if (l == 0) {
 					OFFreeMemory(buffer);
 					return nil;
@@ -244,6 +274,23 @@ parseString(const char **pointer, const char *stop, size_t *line)
 				i += l;
 				*pointer += 11;
 
+				break;
+			case 'x':
+				c1 = parseHexEscape(*pointer - 1, stop);
+				if (c1 == 0xFFFF) {
+					OFFreeMemory(buffer);
+					return nil;
+				}
+
+				l = _OFUTF8StringEncode(c1, buffer + i);
+				if (l == 0) {
+					OFFreeMemory(buffer);
+					return nil;
+				}
+
+				i += l;
+				*pointer += 3;
+
 				break;
 			case '\r':
 				(*pointer)++;
@@ -331,7 +378,7 @@ parseIdentifier(const char **pointer, const char *stop)
 
 			/* Normal character */
 			if ((c1 & 0xFC00) != 0xD800) {
-				l = OFUTF8StringEncode(c1, buffer + i);
+				l = _OFUTF8StringEncode(c1, buffer + i);
 				if (l == 0) {
 					OFFreeMemory(buffer);
 					return nil;
@@ -356,7 +403,7 @@ parseIdentifier(const char **pointer, const char *stop)
 
 			c = (((c1 & 0x3FF) << 10) | (c2 & 0x3FF)) + 0x10000;
 
-			l = OFUTF8StringEncode(c, buffer + i);
+			l = _OFUTF8StringEncode(c, buffer + i);
 			if (l == 0) {
 				OFFreeMemory(buffer);
 				return nil;
