@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -42,13 +42,25 @@
 #endif
 
 #ifdef OF_WII
+# define asm __asm__
 # define nanosleep ogc_nanosleep
+# include <gccore.h>
 # include <ogcsys.h>
 # undef nanosleep
+# undef asm
+#endif
+
+#ifdef OF_NINTENDO_DS
+# define asm __asm__
+# include <nds.h>
+# undef asm
 #endif
 
 #ifdef OF_NINTENDO_3DS
-# include <3ds/svc.h>
+/* Newer versions of libctru started using id as a parameter name. */
+# define id id_3ds
+# include <3ds.h>
+# undef id
 #endif
 
 #import "OFThread.h"
@@ -94,6 +106,7 @@
 # import "OFTLSKey.h"
 # if defined(OF_AMIGAOS) && defined(OF_HAVE_SOCKETS)
 #  import "OFSocket.h"
+#  import "OFSocket+Private.h"
 # endif
 
 static OFTLSKey threadSelfKey;
@@ -127,7 +140,7 @@ callMain(id object)
 
 #if defined(OF_AMIGAOS) && defined(OF_HAVE_SOCKETS)
 	if (thread.supportsSockets)
-		if (!OFSocketInit())
+		if (!_OFSocketInit())
 			@throw [OFInitializationFailedException
 			    exceptionWithClass: thread.class];
 #endif
@@ -155,7 +168,7 @@ callMain(id object)
 
 #if defined(OF_AMIGAOS) && !defined(OF_MORPHOS) && defined(OF_HAVE_SOCKETS)
 	if (thread.supportsSockets)
-		OFSocketDeinit();
+		_OFSocketDeinit();
 #endif
 
 	thread->_running = OFThreadStateWaitingForJoin;
@@ -313,6 +326,23 @@ callMain(id object)
 #endif
 }
 
+#if defined(OF_WII)
++ (void)waitForVerticalBlank
+{
+	VIDEO_WaitVSync();
+}
+#elif defined(OF_NINTENDO_DS)
++ (void)waitForVerticalBlank
+{
+	swiWaitForVBlank();
+}
+#elif defined(OF_NINTENDO_3DS)
++ (void)waitForVerticalBlank
+{
+	gspWaitForVBlank();
+}
+#endif
+
 #ifdef OF_HAVE_THREADS
 + (void)terminate
 {
